import { Injectable } from '@angular/core';
import { FormControl, AbstractControl, ValidatorFn } from '@angular/forms';
import { isIPv4, isIPv6 } from './ip';

const is: any = (window as any).is;
const REGEXES = {
    positiveInt: /^[1-9]\d*$/,
    positiveZTOFloat: /^(0|(0)\.([0-9]{1,2})|1)$/, // 0-1之间的小数
    positiveZTTFloat: /^(\d(\.\d)?|10)$/, // 0-10之间的整数包括小数
    positiveNum: /^(0|[1-9]\d{0,8})$/,// 0-3999999999之间的整数（Number转换精度限制不能超过9007199254740991）
    filePath: /^([\/]{0,1}[A-Za-z0-9\_]+)+$/,
    ipv4: /^(?:(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])$/,
    // tslint:disable-next-line:max-line-length
    ipv6: /^((?=.*::)(?!.*::.+::)(::)?([\dA-F]{1,4}:(:|\b)|){5}|([\dA-F]{1,4}:){6})((([\dA-F]{1,4}((?!\3)::|:\b|$))|(?!\2\3)){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})$/i,
    // tslint:disable-next-line:max-line-length
    url: /^((http[s]{0,1}|ftp):\/\/){0,1}((([0-9]{1,3}\.){3}[0-9]{1,3})|([a-zA-Z0-9\.\-]+\.([a-zA-Z]{2,6})))(:\d+)?(\/[a-zA-Z0-9\-~!@#$%^&*+?:_\/=<>\.]*)?$/i, // 有无协议均可且无文件
    // tslint:disable-next-line:max-line-length
    // email: /^[a-zA-Z0-9]([a-zA-Z0-9]*[-_]?[a-zA-Z0-9]+)*@([a-zA-Z0-9]*[-_]?[a-zA-Z0-9]+)+([\.][a-zA-Z0-9]+)*[\.][a-zA-Z]{2,3}([\.][a-zA-Z]{2})?$/
    email: /^([A-Za-z0-9_\-\.])+\@([A-Za-z0-9_\-\.])+\.([A-Za-z]{2,4})$/
};

@Injectable({
    providedIn: 'root'
})
export class ValidatorService {
    constructor() { }

    // 多个url校验
    static urlMulti(control: FormControl): { [prop: string]: boolean } {
        // console.log('multiUrl', control);
        const value = control.value && control.value.trim();
        if (!value) {
            return;
        }
        /**
         * 支持多个url按照逗号、空格、换行符、分号分割
         * 先将字符串分割，逐个校验，全部通过则通过，一旦有一个不合格则不通过
         */
        const URLs = value.split(/[,; \n]/);
        for (const urlString of URLs) {
            if (urlString.length !== 0) {
                if (!REGEXES.url.test(urlString)) {
                    return { urlMulti: true };
                }
            }
        }
        return;
    }

    static specialcode(control: FormControl): { [prop: string]: {} } {
        if (!control.value) {
            return;
        }
        const special = /["%|=;<>:'\\]/;
        if (special.test(control.value)) {
            return { specialcode: '"%|=;<>:\'\\' };
        }
    }
    // 特殊字符 -- #^@/>
    static specialcodeConfig(control: FormControl): { [prop: string]: {} } {
        if (!control.value) {
            return;
        }
        const special = /[#^@/>]/;
        if (special.test(control.value)) {
            return { specialcode: '#^@/>' };
        }
    }

    // 特殊字符校验并集
    static specialcodeUnion(control: FormControl): { [prop: string]: {} } {
        if (!control.value) {
            return;
        }
        const special = /[%"|=;<>:'\\#^@/]/;
        if (special.test(control.value)) {
            return { specialcode: '%"|=;<>:\'\\#^@/' };
        }
    }

    // 事件名称允许输入分号 ;
    static eventName(control: FormControl): { [prop: string]: {} } {
        if (!control.value) {
            return;
        }
        const special = /[%"|=<>:'\\#^@/]/;
        if (special.test(control.value)) {
            return { specialcode: '%"|=<>:\'\\#^@/' };
        }
    }

    // 校验手机号
    static phonenumber(control: FormControl): { [prop: string]: {} } {
        if (!control.value) {
            return;
        }
        const reg = /^1([38][0-9]|4[579]|5[0-3,5-9]|6[6]|7[0135678]|9[89])\d{8}$/;
        if (!reg.test(control.value)) {
            return { phonenumber: '请输入合法手机号' };
        }
    }

    // 数据源端口
    static port(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (
            !(
                REGEXES.positiveInt.test(control.value) &&
                Number(control.value) > 0 &&
                Number(control.value) <= 65535
            )
        ) {
            return { port: true };
        }
    }

    // 响应码
    static code(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (
            !(
                /^(-|[0-9])\d*$/.test(control.value)
            )
        ) {
            return { code: true };
        }
    }

    // 校验南向管理（A接口）端口
    static aport(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (
            !(
                REGEXES.positiveInt.test(control.value) &&
                Number(control.value) > 0 &&
                Number(control.value) < 65535
            )
        ) {
            return { aport: true };
        }
    }

    // 阈值校验---(0,100)的数字(小数最多为小数点后两位)校验
    static thresholdisRange(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        let ArrMen;
        if (control && control.value) {
            ArrMen = control.value.toString().split('.'); // 截取字符串
            if (ArrMen.length === 2) {
                if (ArrMen[1].length > 2) {
                    return { thresholdisRange: true };
                }
            }
        }
        if (!(Number(control.value) > 0 && Number(control.value) < 100)) {
            return { thresholdisRange: true };
        }
    }

    static trimLength(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (control.value.trim().length === 0) {
            return { trimLength: true };
        }
    }

    // 标准名称1：只能包含英文字母、数字和下划线（以字母开头）
    static standardName1(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        const reg = /^[a-zA-Z][a-zA-Z0-9_]*$/;
        if (!reg.test(control.value)) {
            return { standardName1: true };
        }
    }

    // 正整数
    static positiveInt(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.positiveInt.test(control.value)) {
            return { positiveInt: true };
        }
    }

    // Number()转换安全范围
    static positiveNum(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.positiveNum.test(control.value)) {
            return { positiveNum: true };
        }
    }

    // 安全治理配置项校验--0到1之间的小数
    static positiveFloat(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.positiveZTOFloat.test(control.value)) {
            return { positiveFloat: true };
        }
    }

    // 安全治理配置项校验--0-10之间的整数包括小数
    static positiveIntAndFloat(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.positiveZTTFloat.test(control.value)) {
            return { positiveIntAndFloat: true };
        }
    }

    // url校验
    static url(control: FormControl): { [prop: string]: boolean } {
        const value = control.value && control.value.trim();
        if (!value) {
            return;
        }
        if (!REGEXES.url.test(value)) {
            return { url: true };
        }
    }

    // 单个格式IPV4、IPV6校验
    static ip(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (ValidatorService.ipv4(control) && ValidatorService.ipv6(control)) {
            return { ip: true };
        }
    }

    // 单个格式IPV4校验
    static ipv4(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.ipv4.test(control.value)) {
            return { ipv4: true };
        }
    }

    // 单个格式IPV6校验
    static ipv6(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.ipv6.test(control.value)) {
            return { ipv6: true };
        }
    }

    static ipMulti(control: FormControl): { [prop: string]: boolean } {
        let value = '';
        if (control.value && control.value.value) {
            value = control.value.value && control.value.value.trim();
        } else {
            value = control.value && control.value.trim();
        }
        if (!value) {
            return;
        }
        /**
         * 支持多个IP按照逗号、空格、换行符分割
         * 先将字符串分割，逐个校验，全部通过则通过，一旦有一个不合格则不通过
         */
        const IPs = value.split(/[, \n]/);
        let check = true;
        for (const ipString of IPs) {
            if (!isIPv4(ipString) && !isIPv6(ipString)) {
                check = false;
            }
        }
        if (check) {
            return;
        } else {
            return { ipMulti: true };
        }
    }

    // 文件系统类型数据源的文件路径校验
    static filePath(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!REGEXES.filePath.test(control.value)) {
            return { filePath: true };
        }
    }

    /**
       * 某范围内非负整数校验
       * 若range = [1, 10]，表示范围为大于等于1小于等于10的整数
       */
    static IntRange(range: number[]): ValidatorFn {
        return (control: AbstractControl): { [prop: string]: number[] } => {
            if (!control.value) {
                return;
            }
            const reg = /^(0|([1-9]\d*))$/;
            const fromValue = range[0];
            const toValue = range[1];
            if (!(reg.test(control.value) && Number(control.value) >= fromValue && Number(control.value) <= toValue)) {
                return { IntRange: range };
            }
        };
    }

    /**
     * 邮箱校验
     * seperator为邮箱分隔符，count为邮箱个数限制
     */
    static emails(seperator: string, count: number): ValidatorFn {
        return (control: AbstractControl): { [prop: string]: string } => {
            if (!control.value) {
                return;
            }
            const emailList = control.value.split(seperator);
            if (emailList.length > count) {
                return { emails: `个数不能超过${count}个` };
            }
            for (let i = 0; i < emailList.length; i++) {
                if (!REGEXES.email.test(emailList[i])) {
                    return { emails: '格式有误' };
                }
            }
        };
    }

    /**
     * json校验
     */
    static json(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        if (!ValidatorService.strIsJson(control.value)) {
            return { json: true };
        }
    }

    /**
     * 字符串是否是json
     */
    static strIsJson(str) {
        try {
            const obj = JSON.parse(str);
            if (is.json(obj)) {
                return true;
            } else {
                return false;
            }
        } catch (e) {
            return false;
        }
    }

    /**
     *  多个端口：
     *  1.支持端口段，如：100-200
     *  2.支持按照空格分割
     */
    static isMultiPort = (control: FormControl): { [s: string]: boolean } => {
        let value = control.value;
        if (!value) {
            return;
        }
        value = String(value).trim();
        let reg = /^(\d)+$/;
        const ports = value.split(/[, \n]/); // 按照空格分割
        for (let port of ports) {
            let result: boolean = false;
            if (port.indexOf('-') >= 0) {
                let t = port.split('-');
                if (t.length !== 2) {
                    return { port: true };
                }
                if (Number(t[0]) > Number(t[1])) {
                    return { port: true };
                }
                result = reg.test(t[0]) && parseInt(t[0]) <= 65535 && parseInt(t[0]) > 0 &&
                    reg.test(t[1]) && parseInt(t[1]) <= 65535 && parseInt(t[1]) > 0;
            } else {
                result = reg.test(port) && parseInt(port) <= 65535 && parseInt(port) > 0;
            }
            if (!result) {
                return { port: true };
            }
        }
    }

    // 数字限制
    static multiInt(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        const values = control.value.split(',');
        const positiveInt = /^[0-9]\d*$/;
        for (let i = 0; i < values.length; i++) {
            if (!positiveInt.test(values[i])) {
                return { multiInt: true };
            }
        }
        return;
    }

    // 白名单数字限制，多个整数且包含-1
    static whiteMultiInt(control: FormControl): { [prop: string]: boolean } {
        if (!control.value) {
            return;
        }
        const values = control.value.split(',');
        const positiveInt = /^[0-9]\d*$/;
        for (let i = 0; i < values.length; i++) {
            if (!(positiveInt.test(values[i]) || values[i]  === '-1')) {
                return { multiInt: true };
            }
        }
        return;
    }

    // 数字限制
    static tenNumberLimit(control: FormControl): { [prop: string]: boolean } {
        const positiveInt = /^[0-9]\d*$/;
        if (!control.value) {
            return;
        }
        if (!positiveInt.test(control.value)) {
            return { tenNumberLimit: true };
        }
    }
    specialCode(value: string) {
        if (!value) {
            return {
                specialcode: "\"%|=;<>:'\\",
                result: false
            };
        }
        const special = /["%|=;<>:'\\]/;
        if (special.test(value.trim())) {
            return {
                specialcode: "\"%|=;<>:'\\",
                result: true
            };
        } else {
            return {
                specialcode: "\"%|=;<>:'\\",
                result: false
            };
        }
    }

    validateFloat(value) {
        if (REGEXES.positiveZTOFloat.test(value)) {
            return true;
        } else {
            return false;
        }
    }

    ipMulti(value) {
        if (!value) {
            return false;
        }
        /**
         * 支持多个IP按照逗号、空格、换行符分割
         * 先将字符串分割，逐个校验，全部通过则通过，一旦有一个不合格则不通过
         */
        const IPs = value.split(/[, \n]/);
        for (const ipString of IPs) {
            if (ipString.length !== 0) {
                if (!isIPv4(ipString) && !isIPv6(ipString)) {
                    return true;
                }
            }
        }
        return false;
    }
}
