import { Component, Inject, OnInit, Input, Output, EventEmitter, OnChanges, ChangeDetectionStrategy, SimpleChange } from '@angular/core';
/**
 * 示例：
 * <app-es-pagination [totalCounts]="totalCounts" [(pageIndex)]="pageIndex" [pageSize]="pageSize" (refresh)="refresh($event)">
 * </app-es-pagination>
 *
 * ES查询分页组件，当前页或页大小变化的时候会触发 refresh() 事件；pageIndex 和 pageSize以参数形式传入父组件
 *
 * 父组件中需要配置的属性如下：
 * totalCounts: 0,      // 监听总消息数的变化，触发分页组件刷新分页表块消息总条数
 * pageIndex: 1         // 监听当前页变化
 * pageSize: 10,        // 页大小，默认为10
 * pagesLength: 10      // default:10;定义分页的长度必须为偶数
 * pageSizeOptions: [10, 20, 50, 100]      // 用于控制每页显示多少条 可以不设置
 * isSimple: true       // 简易模式：只展示当前页
 * refresh(evt) {
        this.params.pageIndex = evt.pageIndex;
        this.params.pageSize = evt.pageSize;
        this.getList(this.httpService, this.params); // http请求，获取当前页消息
    }
**/

/*用于初始化分页组件属性，防止父组件没有设置相关属性*/
const dataGridComponentConfig = {
    pageIndex: 1,
    pageSize: 10,                       // 每页显示条数
    totalCounts: 0,
    pagesLength: 10,                    // default:10;定义分页的长度必须为偶数
    pageSizeOptions: [10, 20, 50, 100], // 用于控制每页显示多少条
};

@Component({
    selector: 'app-es-pagination',
    templateUrl: './es-pagination.component.html',
    styleUrls: ['./es-pagination.component.styl']
})
export class EsPaginationComponent implements OnInit, OnChanges {
    pageList: Array<any> = [];

    @Input() totalCounts: number;       // 总页数，默认为0
    @Input() pageSize: number;          // 页大小，默认为10
    @Input() pagesLength: number;       // default:10;定义分页的长度必须为偶数 可以不设置
    @Input() pageSizeOptions: any;      // 用于控制每页显示多少条 可以不设置

    @Input() isSimple: boolean;         // 简易模式：只展示当前页
    @Input() disabledNext: boolean;     // 禁止下一页翻页，默认允许
    @Output() refresh = new EventEmitter<any>(true);   // 将当前页pageIndex和页大小pageSize传入参数列表

    private _pageIndex;
    @Input()
    get pageIndex() {
        return this._pageIndex;
    }
    set pageIndex(value) {
        this._pageIndex = value;
        this.pageIndexChange.emit(this._pageIndex);
    }
    @Output()
    pageIndexChange = new EventEmitter();

    totalPages: number;
    // 改变当前页
    public selectIndexPage(pageNumber): void {
        // pageNumber不能为小数，负数，非数字，不能大于总页数，否则不跳转
        if (pageNumber <= 0 || pageNumber > this.totalPages || isNaN(pageNumber) || String(pageNumber).indexOf('.') > -1) {
            pageNumber = this.pageIndex;
        }
        this.selectNext(pageNumber, false);
        // if (this.pageIndex !== pageNumber ) {
        //     this.pageIndex = pageNumber;
        //     const pageSize = this.pageSize;
        //     this.refresh.emit({ pageIndex: pageNumber, pageSize: pageSize });
        // }
    }

    selectNext(pageNumber, disabledNext) {
        if (this.pageIndex !== pageNumber && !disabledNext) {
            this.pageIndex = pageNumber;
            const pageSize = this.pageSize;
            this.refresh.emit({ pageIndex: pageNumber, pageSize: pageSize });
        }
    }
    selectPre(pageNumber) {
        if (this.pageIndex > pageNumber && pageNumber > 0) {
            this.selectNext(pageNumber, false);
        }
    }

    // 选择页大小pageSize
    public changePageSize(event) {
        const pageSize = event;
        this.refresh.emit({ pageIndex: 1, pageSize: pageSize });
    }

    // 获取分页表块pageList页码数组
    getPagination() {
        this.pageList = [];
        // 获取初始化的pageIndex、totalCounts、pagesLength、pageSizeOptions、pageSize
        this.pageSizeOptions = this.pageSizeOptions ? this.pageSizeOptions : dataGridComponentConfig.pageSizeOptions;
        this.pageSize = this.pageSize ? this.pageSize : dataGridComponentConfig.pageSize;
        this.pageIndex = this.pageIndex ? this.pageIndex : dataGridComponentConfig.pageIndex;
        this.totalCounts = this.totalCounts ? this.totalCounts : dataGridComponentConfig.totalCounts;
        this.pagesLength = this.pagesLength ? this.pagesLength : dataGridComponentConfig.pagesLength;

        this.totalPages = Math.ceil(this.totalCounts / this.pageSize);

        const totalPages = this.totalPages;

        let pageIndex = this.pageIndex;
        let pagesLength = this.pagesLength;
        if (pagesLength % 2 === 1) {
            pagesLength += 1;
        }
        if (pageIndex < 1) {
            pageIndex = pageIndex;
        }
        if (totalPages > 0 && pageIndex > totalPages) {
            pageIndex = pageIndex;
        }

        if (totalPages <= pagesLength) {
            // 判断总页数如果小于等于分页的长度，若小于则直接显示
            for (let i = 1; i <= totalPages; i++) {
                this.pageList.push(i);
            }
        } else {
            // 总页数大于分页长度（此时分为三种情况：1.当前页 < pagesLength/2 2. 当前页 > pagesLength/2）
            // 计算中心偏移量
            const offset = Math.ceil(pagesLength / 2); // 向下取整
            if (pageIndex <= offset + 1) {
                for (let i = 1; i <= pagesLength; i++) {
                    this.pageList.push(i);
                }
            } else if (pageIndex > offset + 1) {
                const left = [], right = [];
                if (pageIndex + offset - 1 <= totalPages) {
                    // 右边序列最后一个页码数 <= 总页数
                    for (let i = 0; i < offset; i++) {
                        left.push(pageIndex - offset + i); // 当前页左边的序列
                    }
                    for (let i = pageIndex; i <= pageIndex + offset - 1; i++) {
                        right.push(i);
                    }
                } else {
                    // 右边序列最后一个页码数 > 总页数
                    for (let i = totalPages - pagesLength + 1; i <= totalPages; i++) {
                        left.push(i); // 当前页左边的序列
                    }
                }
                this.pageList = [...left, ...right];
            }
        }
    }

    constructor() {

    }

    ngOnChanges(changes: { [propKey: string]: SimpleChange }): void {
        const pageIndex = changes['pageIndex'];
        const totalCounts = changes['totalCounts'];
        const pageSize = changes['pageSize'];
        if (pageIndex || totalCounts || pageSize) {
            this.getPagination();
        }
    }

    ngOnInit() {
    }

}
