import { Injectable } from '@angular/core';
import { FormControl, ValidatorFn, AbstractControl } from '@angular/forms';

const EventREGEXES = {
  positiveInt: /^[1-9]\d*$/,
  // Ipv4 校验
  ipv4: /^(?:(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}(?:\d|[1-9]\d|1\d{2}|2[0-4]\d|25[0-5])$/,
  //  Ipv6 校验
  // tslint:disable-next-line:max-line-length
  ipv6: /^((?=.*::)(?!.*::.+::)(::)?([\dA-F]{1,4}:(:|\b)|){5}|([\dA-F]{1,4}:){6})((([\dA-F]{1,4}((?!\3)::|:\b|$))|(?!\2\3)){2}|(((2[0-4]|1\d|[1-9])?\d|25[0-5])\.?\b){4})$/i,
  // ipv4 子网掩码校验
  ipv4Mask: /^((\d{1,2}|1\d\d|2[0-4]\d|25[0-5])(\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])){3})?$/,
  // 域名校验
  // domain: /^[a-zA-Z0-9][a-zA-Z0-9-]{1,61}[a-zA-Z0-9](?:\.[a-zA-Z]{2,})+$/,
  domain: /^([0-9a-z-]{0,}\.)?[0-9a-z-]{1,}.([0-9a-z-]{2,}.)?[a-z]{2,}$/,
  // url 有无协议均可且无文件
  // tslint:disable-next-line:max-line-length
  url: /^((http[s]{0,1}|ftp):\/\/){0,1}((([0-9]{1,3}\.){3}[0-9]{1,3})|([a-zA-Z0-9\.\-]+\.([a-zA-Z]{2,6})))(:\d+)?(\/[a-zA-Z0-9\-~!@#$%^&*+?:_\/=<>\.]*)?$/i,
  // 端口校验
  port:  /^(\d)+$/
};

@Injectable({
  providedIn: 'root'
})
export class EventValidateService {

  constructor() { }

  /**
   * 自定义 tab 名称特殊校验
   */
  static visTabName(): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
     // const value = control.value && control.value.trim().length;
      if (!control.value) { return; }

      const blank = /\s+/g;
      if (blank.test(control.value)) {
        return { tab: '不能输入空格'};
      }

      if (control.value === '待处理事件' || control.value === '全部事件' || control.value === '漏洞运维') {
        return {tab: '不能同内置页签同名'};
      }

      const special = /[%"|=;<>:'\\#^@/]/;
      if (special.test(control.value)) {
        return { specialcode: '%"|=;<>:\'\\#^@/' };
      }
    };
  }

  // 校验空格 ---- 此处校验后期需要改动
  static filterBlank(control: FormControl): { [prop: string]: {} } {
    if (!control.value) {
      return;
    }
    const obj = control.value.replace(/^\s+|\s+$/g, '');
    if (!obj) {
      return { fiterBlank: '不能输入空格' };
    }
  }

  /**
   *  多个 url校验
   *  seperator为邮箱分隔符，count为邮箱个数限制
   */
  static moreUrl(seperator: string, count: number): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
      const value = control.value && control.value.trim();
      if (!value) {
        return;
      }
      const urlList = value.split(seperator);
      if (urlList.length > count) {
        return { url: `个数不能超过${count}个` };
      }

      for (let i = 0; i < urlList.length; i++) {
        if (!EventREGEXES.url.test(urlList[i])) {
          return { url: '格式有误' };
        }
      }
    };
  }

  /**
   * 域名封堵： 支持多个 IP ( IPv4, IPv6  ) 或域名，且字符在 10 个范围内
   * seperator为分隔符，count为个数限制
   */
  static blockDomian(seperator: string, count: number): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
      const value = control.value;
      if (!value && value !== '') {
        return;
      }

      const inputList = value.split(seperator);
      if (inputList.length > count) {
        return { domain: `个数不能超过${count}个` };
      }

      for (let i = 0; i < inputList.length; i++) {
        const regDomian = EventREGEXES.domain.test(inputList[i]);
        const regIpv4 = EventREGEXES.ipv4.test(inputList[i]);
        const regIpv6 = EventREGEXES.ipv6.test(inputList[i]);

        if (!regDomian && !regIpv4 && !regIpv6) {
          return { domain: '输入格式非法' };
        }
      }
    };
  }

    /**
   * IP 封堵：  IP ( IPv4, IPv6 )或 IP ( IPv4, IPv6  ) 网段
   * IP 网段只支持 B 和 C 网段，且 IP 和 IP 网段不能混合输入
   * 支持多个值之间以英文逗号分隔，不超过 10 个。
   * seperator为分隔符，count为个数限制
   */
  static blockIp(seperator: string, count: number): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
      const value = control.value && control.value.trim();
      if (!value) {
        return;
      }

      const inputList = value.split(seperator);
      if (inputList.length > count) {
        return { ip: `响应对象个数不能超过${count}个` };
      }
      const allIpTypes = [];  // 用于判断网段和IP是否混合输入
      for (let i = 0; i < inputList.length; i++) {
        const regIpv4 = EventREGEXES.ipv4.test(inputList[i]);
        const regIpv6 = EventREGEXES.ipv6.test(inputList[i]);
        const regSegIpv4 = EventValidateService.isIpv4Segment(inputList[i], 23, 32);
        // const regSegIpv6 = EventValidateService.isIpv6Segment(inputList[i], 16, 32);

        // if (!regIpv4 && !regSegIpv4 && !regIpv6 && !regSegIpv6) {
        //   return { ip: '输入格式非法' };
        // }
        const inputItem = inputList[i].split('.');
        let inputIsItem;

        if (inputItem[0] > 223 || inputItem[0] < 128) {
           inputIsItem = false;
        } else {
          inputIsItem = true;
        }

        if (inputList.length <= 1) {
          if (value.indexOf('/') !== -1) {
            if (!inputIsItem) {
              return { ip: 'IPv4 网段只支持 B,C 网段' };
            } else if (!regSegIpv4 || regIpv4 || regIpv6) {
              return { ip: 'IP 网段格式不合法' };
            }
          } else {
            if (!regIpv4 && !regIpv6) {
              return { ip: 'IP 格式不合法' };

            }
          }
        } else {
          if (value.indexOf('/') !== -1) {
            if (!regSegIpv4 || regIpv4 || regIpv6) {
              return { ip: '多个封堵对象，IP 和网段不可混合输入' };
            }
          } else {
            if (!regIpv4 && !regIpv6) {
              return { ip: '多个封堵对象，IP 和网段不可混合输入' };
            }
          }
        }
      }
      // 判断网段和IP是否混合输入
      if (allIpTypes.length > 1) {
        if (allIpTypes.join('').indexOf('0') > -1 && allIpTypes.join('').indexOf('1') > -1) {
          return { ip: '多个响应对象，IP 和网段不可混合输入' };
        }
      }
    };
  }

   /**
   * 会话封堵： 会话三元组【 源 IP ( IPv4, IPv6  )、目的 IP ( IPv4, IPv6 ) 和目的端口】
   * 192.168.1.1|192.168.1.2|445
   * 不支持多值输入
   */
  static blockSession(): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
      const value = control.value;
      if (!value && value !== '') {
        return;
      }

      const arr = value.split('|');
      const arr0Ipv4 =  EventREGEXES.ipv4.test(arr[0]);
      const arr1Ipv4 =  EventREGEXES.ipv4.test(arr[1]);
      const arr0Ipv6 =  EventREGEXES.ipv6.test(arr[0]);
      const arr1Ipv6 =  EventREGEXES.ipv6.test(arr[1]);
      const arr2Port =  EventValidateService.isPort(arr[2]);

      if ((arr.length !== 3) || (!arr0Ipv4 && !arr0Ipv6) || (!arr1Ipv4 && !arr1Ipv6)) {
        return { session: '请输入正确会话三元组' };
      }
      if (!arr2Port) {
        return { session: '请输入正确端口 ( 1 ~ 65535 )' };
      }
      if ((arr0Ipv4 && arr1Ipv6) || (arr1Ipv4 && arr0Ipv6)) {
        return { session: '会话三元组不支持 IPv4，IPv6 混合输入' };
      }
    };
  }

  /**
   * 流量牵引： 支持多个 IP ( IPv4, IPv6  ) ，且在 10 个范围内
   * seperator为分隔符，count为个数限制
   */
  static blockFlow(seperator: string, count: number): ValidatorFn {
    return (control: AbstractControl): { [prop: string]: string } => {
      const value = control.value;
      if (!value && value !== '') {
        return;
      }

      const inputList = value.split(seperator);
      if (inputList.length > count) {
        return { flow: `个数不能超过${count}个` };
      }

      for (let i = 0; i < inputList.length; i++) {
        const regIpv4 = EventREGEXES.ipv4.test(inputList[i]);
        const regIpv6 = EventREGEXES.ipv6.test(inputList[i]);

        if (!regIpv4 && !regIpv6) {
          return { flow: '输入格式非法' };
        }
      }
    };
  }

  /**
   *  waf  1440
   * other 43200
   */
  // 正整数
  static blockTime(control: FormControl): { [prop: string]: string } {
      if (!control.value && control.value !== '') {
          return;
      }

      if (!(
        EventREGEXES.positiveInt.test(control.value) &&
        Number(control.value) > 0 &&
        Number(control.value) <= 1440
      )) {
        return { time: '请输入正整数，时间不超过 1440 分钟' };
    }
  }

  /**
   * 网段校验
   * 192.168.1.1/18
   * 2000:0:0:0:0:0:0:1/20
   * min, max 为网段范围
   */
    // 是否为网段格式IPV4 --- string
   static isIpv4Segment(ipSegment, min, max) {
      const arr = ipSegment.split('/');
      if (
        arr.length === 2 &&
        EventREGEXES.ipv4.test(arr[0]) &&
        Number(arr[1]) > min &&
        Number(arr[1]) <= max
      ) {
        return true;
      } else {
        return false;
      }
    }

    // 是否为网段格式IPV6 --- string
   static isIpv6Segment(ipSegment, min, max) {
      const arr = ipSegment.split('/');
      if (
        arr.length === 2 &&
        EventREGEXES.ipv6.test(arr[0]) &&
        Number(arr[1]) > min &&
        Number(arr[1]) <= max
      ) {
        return true;
      } else {
        return false;
      }
    }

   // 端口校验
   static isPort(port) {
     if (
      EventREGEXES.port.test(port) &&
      parseInt(port, 0) <= 65535 &&
      parseInt(port, 0) >= 0
     ) {
      return true;
     } else {
      return false;
    }
   }

   // 基础模糊查询特殊字符
   static fuzzySpecode(control: FormControl): { [prop: string]: {} } {
    if (!control.value) {
        return;
    }
    const special = /["|<>']/;
    if (special.test(control.value)) {
        return { specialcode: '"|<>\'\\' };
    }
  }

  // 关联事件及日志 URL 特殊字符
  static linkUrlSpeCode(control: FormControl): { [prop: string]: {} } {
    if (!control.value) {
        return;
    }
    const special = /["|;<>'\\]/;
    if (special.test(control.value)) {
        return { specialcode: '"|;<>\'\\' };
    }
  }
}
